// ***************************************************************************************
//                                         DATABASE
// ***************************************************************************************

#include "denise.h"
#include "denise_protos.h"

#define DENISE_VERSION_CHECK "$VERSION=5.1"

// deklaracje prywatne
// -

// ***************************************************************************************
// EXTERN                            BOOL dat_load_all(void)                         AMIGA
// ***************************************************************************************

BOOL dat_load_all(struct DeniseHandle* handle)
{
	handle->cache.denise=NULL;

	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LOADING_DATA,"wczytuje dane..."));

	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LANGUAGE_DATA,"* language..."));
	if (dat_language_load(handle)) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LOADED,"loaded"));
	else return FAIL;

	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LOGIC_DATA,"* logic..."));
	if (dat_logic_load(handle)) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LOADED,"loaded"));
	else return FAIL;

	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_MEANINGS_DATA,"* meanings..."));
	if (dat_meanings_load(handle)) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LOADED,"loaded"));
	else return FAIL;

	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_FACTS_DATA,"* facts..."));
	if (!dat_facts_load(handle)) return FAIL;

	handle->cache.last_list=NULL;
	handle->cache.identify_mode=FALSE;
	handle->cache.explain_mode=FALSE;
	handle->cache.temp_fact=NULL;
	handle->cache.facts=NULL;
	handle->cache.user=NULL;
	handle->cache.theme=NULL;

	return TRUE;
}


// ***************************************************************************************
// EXTERN                            BOOL dat_save_all(void)                         AMIGA
// ***************************************************************************************

BOOL dat_save_all(struct DeniseHandle* handle)
{
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_SAVING_DATA,"zapisuje dane..."));
	dat_language_save(handle);
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_MEANINGS_DATA,"* meanings..."));
	dat_meanings_save(handle);

	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_FACTS_DATA,"* facts..."));
	dat_facts_save(handle);

	return TRUE;
}


// ***************************************************************************************
// EXTERN                            BOOL dat_erase_all(void)                        OTHER
// ***************************************************************************************

BOOL dat_erase_all(struct DeniseHandle* handle)
{
//	struct Fact *fact,*fact_next;

//return TRUE;

	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_ERASING_DATA,"kasuje dane z pamieci"));

   DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LANGUAGE_DATA,"* language... "));
   dat_language_erase(handle);

   DeniseReport(handle,UNN_GetLocaleString(handle,MSG_FACTS_DATA,"* facts... "));
   dat_facts_erase(handle);

   DeniseReport(handle,UNN_GetLocaleString(handle,MSG_MEANINGS_DATA,"* meanings... "));
   dat_meanings_erase(handle);

   DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LOGIC_DATA,"* logic... "));
   dat_logic_erase(handle);

   if (handle->cache.last_list!=NULL)
   {
      log_clauses_free(&(handle->cache.last_list));
//      handle->cache.last_list=NULL;
   }

   if (handle->cache.temp_fact!=NULL)
   {
      dat_fact_free(handle->cache.temp_fact);
		free(handle->cache.temp_fact);
      handle->cache.temp_fact=NULL;
   }

	if (handle->cache.facts!=NULL)
   {
		dat_fact_free(handle->cache.facts);
		free(handle->cache.facts);
        handle->cache.facts=NULL;
	}

	return TRUE;
}


// ***************************************************************************************
// EXTERN                       ULONG dat_file_open(STRPTR ,STRPTR )
// ***************************************************************************************

FILE *dat_file_open(struct DeniseHandle* handle, char* file, char* mode)
{
	FILE *stream;
	char text[50];
	if (file)
	{
		if (mode)
		{
			if (mode[0]=='a')
			{
				if ((stream=fopen(file,"r")))
				{
					// kontrola wersji pliku - wczytujemy pierwsza linie
			      if (glo_fgets(text,50,stream)==NULL)
					{
						fclose(stream);
						if ((stream=fopen(file,"w"))==NULL) return NULL;
						// jesli nie ma wersji, to trzeba dopisac
						fprintf(stream,"%s\n",DENISE_VERSION_CHECK);
						fclose(stream);
					}
					else
					{
						fclose(stream);
						text[max(strlen(text)-1,0)]='\0';

						if (strcmp(text,DENISE_VERSION_CHECK)!=0)
						{
							DeniseReport(handle,UNN_GetLocaleString(handle,MSG_FILE_VERSION_ERROR,"bledna wersja pliku: %s (jest: %s, ma byc: %s)"),file,text,DENISE_VERSION_CHECK);
							return NULL;
						}
					}
				}
				else
				{
					if ((stream=fopen(file,"w"))==NULL) return NULL;
					// jesli nie ma wersji, to trzeba dopisac
					fprintf(stream,"%s\n",DENISE_VERSION_CHECK);
					fclose(stream);
				}
			}

			if ((stream=fopen(file,mode)))
			{
				if (mode[0]=='r')
				{
					// kontrola wersji pliku - wczytujemy pierwsza linie
			      if (glo_fgets(text,50,stream)==NULL)
					{
						fclose(stream);
						return NULL;
					}

					text[max(strlen(text)-1,0)]='\0';

					if (strcmp(text,DENISE_VERSION_CHECK)!=0)
					{
						DeniseReport(handle,UNN_GetLocaleString(handle,MSG_FILE_VERSION_ERROR,"bledna wersja pliku: %s (jest: %s, ma byc: %s)"),file,text,DENISE_VERSION_CHECK);
						fclose(stream);
						return NULL;
					}
				}
				else if (mode[0]=='w')
				{
					fprintf(stream,"%s\n",DENISE_VERSION_CHECK);
				}

				return stream;
			}
			handle->ErrorCode=UNN_DosError(stream);
		} else handle->ErrorCode=ERR_SYS_INVALIDPARAM;
	} else handle->ErrorCode=ERR_FILE_INVALIDFILENAME;
	return NULL;
}



// ***************************************************************************************
// EXTERN                       ULONG dat_file_move(STRPTR ,STRPTR)
// ***************************************************************************************

BOOL dat_file_move(struct DeniseHandle* handle, char* src,char* dst)
{
	if (dst)
	{
		if (src)
		{
			if (remove(dst)==0)
			{
				if (rename(src,dst)==0)
				{
					return TRUE;
				}
			}
			handle->ErrorCode=UNN_DosError(NULL);
		} else handle->ErrorCode=ERR_FILE_INVALIDFILENAME;
	} else handle->ErrorCode=ERR_FILE_INVALIDFILENAME;
	return FAIL;
}
