// ***************************************************************************************
//                                            FACTS
// ***************************************************************************************

#include "denise.h"
#include "denise_protos.h"

// deklaracje prywatne
static BOOL database_load(struct DeniseHandle* handle,char *filename);

// ***************************************************************************************
// EXTERN                    BOOL dat_facts_load(struct DeniseHandle*)
// ***************************************************************************************
BOOL dat_facts_load(struct DeniseHandle* handle)
{
   // funkcja wczytuje wszystkie bazy faktow podane w pliku

   FILE *stream=NULL;
   char temp[100];
   int nr1;
   char *factsdirfile=NULL;

   handle->facts=NULL;
   asprintf(&factsdirfile,"%s%s%s",handle->Prefix,handle->FactsDir,FACTS_LIST_FILE);
   if ((stream=dat_file_open(handle,factsdirfile,"r"))==NULL) {
	   free(factsdirfile);
	   return TRUE;
   }
   if (glo_file_lenght(stream)==0) {fclose(stream);return TRUE;}

   for (;;)
   {
      if (glo_fgets(temp,200,stream)==NULL) break;
      temp[strlen(temp)-1]='\0';
		DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LOADING_FACTS_DATABASE,"wczytuje baze: %s"),temp);
      database_load(handle,temp);
		DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LOADED,"loaded"));
   }
   fclose(stream);

   // przygotowanie danych do wykorzystania
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_PREPARING_DATABASE,"konfiguracja wczytanych danych..."));
   fac_prepare_databases(handle);

   // odszukanie informacji o osobowosci Denise
   nr1=ext_set_denise_theme(handle);
   if (nr1==1) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_DENISE_THEME_FOUNDED,"odnaleziono temat DENISE"));
   else if (nr1>1)
   {
		DeniseReport(handle,UNN_GetLocaleString(handle,MSG_TOO_MUCH_DENISE_THEMES,"odnaleziono zbyt wiele tematow DENISE"));
		DeniseReport(handle,UNN_GetLocaleString(handle,MSG_ERASING_FACTS_DATABASES,"kasuje z pamieci bazy faktow"));
      dat_facts_erase(handle);
   }
   else if (nr1==0) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_DENISE_THEME_NOT_FOUND,"nie odnaleziono tematu DENISE"));

   return TRUE;
}

static BOOL database_load(struct DeniseHandle* handle,char *filename)
{
   // funkcja wczytuje pojedyncza baze faktow

   int nr1;
   char temp[1000];
   FILE *stream=NULL;
   struct FactDatabase *database;
   struct Theme *theme,*theme_act=NULL;
   struct Fact *fact,*fact_act=NULL;
   struct FactSimple *fs,*fs_act=NULL;

   sprintf(temp,"%s%s%s.dat",handle->Prefix,handle->FactsDir,filename);
   if ((stream=dat_file_open(handle,temp,"r"))==NULL) return TRUE;
   if (glo_file_lenght(stream)==0) {fclose(stream);return TRUE;}

   // czy to poczatek bazy?
   if (glo_fgets(temp,200,stream)==NULL) goto blad;
   temp[strlen(temp)-1]='\0';
   if (strcmp(temp,DATABASE_START)!=0) goto blad;

   // baza danych
   database=malloc(sizeof(struct FactDatabase));
	if (database==NULL)
	{
printf("database2.c ERROR 1\n");
return FALSE;
	}
   database->next=NULL;
   database->themes=NULL;
   database->facts=NULL;
   database->name=NULL;
   database->mode_name1=NULL;
   database->mode_name2=NULL;
   database->mode_name3=NULL;

   database->name=malloc(strlen(filename)+1);
	if (database->name==NULL)
	{
printf("database2.c ERROR 2\n");
return FALSE;
	}
   memcpy(database->name,filename,strlen(filename)+1);

   // 1-ACCEPT_NEW_THEMES, 2-NO_NEW_THEMES
   if (glo_fgets(temp,200,stream)==NULL) goto blad;
   temp[strlen(temp)-1]='\0';
   memmove(temp,temp+3,strlen(temp)-2);
   database->mode_name1=malloc(strlen(temp)+1);
	if (database->mode_name1==NULL)
	{
printf("database2.c ERROR 3\n");
return FALSE;
	}
   memcpy(database->mode_name1,temp,strlen(temp)+1);
   database->new_themes=atoi(temp)==1 ? TRUE : FALSE;

   // 1-ACCEPT_NEW_FACTS, 2-NO_NEW_FACTS
   if (glo_fgets(temp,200,stream)==NULL) goto blad;
   temp[strlen(temp)-1]='\0';
   memmove(temp,temp+3,strlen(temp)-2);
   database->mode_name2=malloc(strlen(temp)+1);
	if (database->mode_name2==NULL)
	{
printf("database2.c ERROR 4\n");
return FALSE;
	}
   memcpy(database->mode_name2,temp,strlen(temp)+1);
   database->new_facts=atoi(temp)==1 ? TRUE : FALSE;

   // 1-ACCEPT_FORWARD_THEMES, 2-NO_FORWARD_THEMES
   if (glo_fgets(temp,200,stream)==NULL) goto blad;
   temp[strlen(temp)-1]='\0';
   memmove(temp,temp+3,strlen(temp)-2);
   database->mode_name3=malloc(strlen(temp)+1);
	if (database->mode_name3==NULL)
	{
printf("database2.c ERROR 5\n");
return FALSE;
	}
   memcpy(database->mode_name3,temp,strlen(temp)+1);
   database->fwd_themes=atoi(temp)==1 ? TRUE : FALSE;

   // unikalny numer identyfikacyjny
   if (glo_fgets(temp,200,stream)==NULL) goto blad;
   temp[strlen(temp)-1]='\0';
   memmove(temp,temp+3,strlen(temp)-2);
   strcpy(database->databaseID,temp);

   // wczytujemy kolejne tematy
   for (;;)
   {
      if (glo_fgets(temp,200,stream)==NULL) goto blad;
      temp[strlen(temp)-1]='\0';
      if (strstr(temp,THEME_START)!=NULL)
      {
         // temat
         theme=malloc(sizeof(struct Theme));
			if (theme==NULL)
			{
printf("database2.c ERROR 6\n");
return FALSE;
			}
         theme->next=NULL;
         theme->identify=NULL;
         theme->database=database;
         theme->number=0;
         theme->fwd_databaseID[0]='\0';
         theme->fwd_number=0;

         // identyfikatory tematu
         if (glo_fgets(temp,200,stream)==NULL) goto blad;
         temp[strlen(temp)-1]='\0';
         memmove(temp,temp+6,strlen(temp)-5);
         theme->identify=malloc(strlen(temp)+1);
			if (theme->identify==NULL)
			{
printf("database2.c ERROR 7\n");
return FALSE;
			}
         memcpy(theme->identify,temp,strlen(temp)+1);

         // numer identyfikacyjny tematu
         if (glo_fgets(temp,200,stream)==NULL) goto blad;
         temp[strlen(temp)-1]='\0';
         memmove(temp,temp+6,strlen(temp)-5);
         theme->number=atoi(temp);

         // dopisanie tematu do bazy
         if (database->themes==NULL) database->themes=theme;
         else theme_act->next=theme;
         theme_act=theme;

         // pobieramy oczekiwany znak zakonczenia tematu
         if (glo_fgets(temp,200,stream)==NULL) goto blad;
         temp[strlen(temp)-1]='\0';

         if (strstr(temp,THEME_END)==NULL) goto blad;
      }
      else if (strstr(temp,FACT_START)!=NULL) break;
      else goto blad;
   }

   // wczytujemy kolejne fakty
   for (;;)
   {
      if (strstr(temp,FACT_START)!=NULL)
      {
         // fakt
         fact=malloc(sizeof(struct Fact));
			if (fact==NULL)
			{
printf("database2.c ERROR 7\n");
return FALSE;
			}
         fact->next=NULL;
         fact->ile_klauzul=0;
         fact->logic=NULL;
         fact->fs=NULL;

         // wczytujemy dane faktu: czas
         if (glo_fgets(temp,200,stream)==NULL) goto blad;
         temp[strlen(temp)-1]='\0';
         memmove(temp,temp+6,strlen(temp)-5);
         memcpy(fact->time,temp,strlen(temp)+1);

         // logika: liczba klauzul
         if (glo_fgets(temp,200,stream)==NULL) goto blad;
         temp[strlen(temp)-1]='\0';
         memmove(temp,temp+6,strlen(temp)-5);
         fact->ile_klauzul=atoi(temp);

         fact->logic=(char **) malloc(fact->ile_klauzul*sizeof(char *));
			if (fact->logic==NULL)
			{
printf("database2.c ERROR 8\n");
return FALSE;
			}
         for (nr1=0;nr1<fact->ile_klauzul;nr1++)
         {
            if (glo_fgets(temp,200,stream)==NULL) goto blad;
            temp[strlen(temp)-1]='\0';
            memmove(temp,temp+6,strlen(temp)-5);
            fact->logic[nr1]=malloc(strlen(temp)+1);
				if (fact->logic[nr1]==NULL)
				{
printf("database2.c ERROR 9\n");
return FALSE;
				}
            memcpy(fact->logic[nr1],temp,strlen(temp)+1);
         }

         // wczytujemy kolejne fakty proste
         for (;;)
         {
            if (glo_fgets(temp,200,stream)==NULL) goto blad;
            temp[strlen(temp)-1]='\0';

            if (strstr(temp,FACT_SIMPLE_START)!=NULL)
            {
               // fakt prosty
               fs=malloc(sizeof(struct FactSimple));
               fs->next=NULL;
               fs->databaseID[0]='\0';
               fs->theme_number=0;
               fs->sentence=malloc(sizeof(struct FSentence));

               // wczytujemy dane faktu prostego: jakosc
               if (glo_fgets(temp,200,stream)==NULL) goto blad;
               temp[strlen(temp)-1]='\0';
               memmove(temp,temp+9,strlen(temp)-8);
               fs->quality=atoi(temp);

               // wczytujemy dane faktu prostego: unikalny numer identyfikacyjny bazy danych do ktorej nalezy temat
               if (glo_fgets(temp,200,stream)==NULL) goto blad;
               temp[strlen(temp)-1]='\0';
               memmove(temp,temp+9,strlen(temp)-8);
               strcpy(fs->databaseID,temp);

               // wczytujemy dane faktu prostego: numer tematu
               if (glo_fgets(temp,200,stream)==NULL) goto blad;
               temp[strlen(temp)-1]='\0';
               memmove(temp,temp+9,strlen(temp)-8);
               fs->theme_number=atoi(temp);

               if (glo_fgets(temp,200,stream)==NULL) goto blad;
               temp[strlen(temp)-1]='\0';
               if (strstr(temp,INFORMATION_START)==NULL) goto blad;

               // INFORMACJA:
               if (glo_fgets(temp,200,stream)==NULL) goto blad;
               temp[strlen(temp)-1]='\0';
               memmove(temp,temp+12,strlen(temp)-11);
               fs->sentence->language=malloc(strlen(temp)+1);
               memcpy(fs->sentence->language,temp,strlen(temp)+1);

               if (glo_fgets(temp,200,stream)==NULL) goto blad;
               temp[strlen(temp)-1]='\0';
               memmove(temp,temp+12,strlen(temp)-11);
               if ((fs->sentence->id_subject=malloc(strlen(temp)+1))==NULL) goto blad;
               memcpy(fs->sentence->id_subject,temp,strlen(temp)+1);

               if (glo_fgets(temp,200,stream)==NULL) goto blad;
               temp[strlen(temp)-1]='\0';
               memmove(temp,temp+12,strlen(temp)-11);
               if (strlen(temp)==0) fs->sentence->id_global=NULL;
               else
               {
                  if ((fs->sentence->id_global=malloc(strlen(temp)+1))==NULL) goto blad;
                  memcpy(fs->sentence->id_global,temp,strlen(temp)+1);
               }

               if (glo_fgets(temp,200,stream)==NULL) goto blad;
               temp[strlen(temp)-1]='\0';
               memmove(temp,temp+12,strlen(temp)-11);
               if ((fs->sentence->orig=malloc(strlen(temp)+1))==NULL) goto blad;
               memcpy(fs->sentence->orig,temp,strlen(temp)+1);

               if (glo_fgets(temp,200,stream)==NULL) goto blad;
               temp[strlen(temp)-1]='\0';
               memmove(temp,temp+12,strlen(temp)-11);
               if ((fs->sentence->words=malloc(strlen(temp)+1))==NULL) goto blad;
               memcpy(fs->sentence->words,temp,strlen(temp)+1);

               if (glo_fgets(temp,200,stream)==NULL) goto blad;
               temp[strlen(temp)-1]='\0';
               memmove(temp,temp+12,strlen(temp)-11);
               fs->sentence->base_words=malloc(strlen(temp)+1);
               memcpy(fs->sentence->base_words,temp,strlen(temp)+1);
//               fs->sentence->ile_fleks=glo_count_parts(handle,temp,',');

               if (glo_fgets(temp,200,stream)==NULL) goto blad;
               temp[strlen(temp)-1]='\0';
               memmove(temp,temp+12,strlen(temp)-11);
               fs->sentence->voc=malloc(strlen(temp)+1);
               memcpy(fs->sentence->voc,temp,strlen(temp)+1);
               fs->sentence->ile_fleks=glo_count_parts(handle,temp,',');

               if (glo_fgets(temp,200,stream)==NULL) goto blad;
               temp[strlen(temp)-1]='\0';
               memmove(temp,temp+12,strlen(temp)-11);
					if (strlen(temp)==0) fs->sentence->group_voc=NULL;
					else
					{
	               fs->sentence->group_voc=malloc(strlen(temp)+1);
   	            memcpy(fs->sentence->group_voc,temp,strlen(temp)+1);
					}

               if (glo_fgets(temp,200,stream)==NULL) goto blad;
               temp[strlen(temp)-1]='\0';
               memmove(temp,temp+12,strlen(temp)-11);
               fs->sentence->words_number=strlen(temp);
               fs->sentence->zrost_poz=malloc(strlen(temp)+1);
               for (nr1=0;nr1<fs->sentence->words_number;nr1++) fs->sentence->zrost_poz[nr1]=temp[nr1]-'0';

               if (glo_fgets(temp,200,stream)==NULL) goto blad;
               temp[strlen(temp)-1]='\0';
               memmove(temp,temp+12,strlen(temp)-11);
               fs->sentence->priority=malloc(strlen(temp)+1);
               for (nr1=0;nr1<fs->sentence->words_number;nr1++) fs->sentence->priority[nr1]=temp[nr1]-'0';

               if (glo_fgets(temp,200,stream)==NULL) goto blad;
               temp[strlen(temp)-1]='\0';
               memmove(temp,temp+12,strlen(temp)-11);
               fs->sentence->fleksyjny=malloc(strlen(temp)+1);
               for (nr1=0;nr1<fs->sentence->words_number;nr1++) fs->sentence->fleksyjny[nr1]=temp[nr1]-'0';

               if (glo_fgets(temp,200,stream)==NULL) goto blad;
               temp[strlen(temp)-1]='\0';
               memmove(temp,temp+12,strlen(temp)-11);
               fs->sentence->member=malloc(strlen(temp)+1);
               for (nr1=0;nr1<fs->sentence->words_number;nr1++) fs->sentence->member[nr1]=temp[nr1]-'0';

               if (glo_fgets(temp,200,stream)==NULL) goto blad;
               temp[strlen(temp)-1]='\0';
               memmove(temp,temp+12,strlen(temp)-11);
               fs->sentence->groups_number=strlen(temp);
               fs->sentence->parent=malloc(strlen(temp)+1);
               for (nr1=0;nr1<fs->sentence->groups_number;nr1++) fs->sentence->parent[nr1]=temp[nr1]-'0';

               if (glo_fgets(temp,200,stream)==NULL) goto blad;
               temp[strlen(temp)-1]='\0';
               memmove(temp,temp+12,strlen(temp)-11);
//               fs->sentence->groups_number=strlen(temp);
               fs->sentence->parent_word=malloc(strlen(temp)+1);
               for (nr1=0;nr1<fs->sentence->groups_number;nr1++) fs->sentence->parent_word[nr1]=temp[nr1]-'0';

               if (glo_fgets(temp,200,stream)==NULL) goto blad;
               temp[strlen(temp)-1]='\0';
               memmove(temp,temp+12,strlen(temp)-11);
               fs->sentence->function=malloc(strlen(temp)+1);
               for (nr1=0;nr1<fs->sentence->groups_number;nr1++) fs->sentence->function[nr1]=temp[nr1]-'0';

               if (glo_fgets(temp,200,stream)==NULL) goto blad;
               temp[strlen(temp)-1]='\0';
               memmove(temp,temp+12,strlen(temp)-11);
               fs->sentence->posneg=malloc(strlen(temp)+1);
               for (nr1=0;nr1<fs->sentence->groups_number;nr1++) fs->sentence->posneg[nr1]=temp[nr1]-'0';

               // koniec informacji
               if (glo_fgets(temp,200,stream)==NULL) goto blad;
               temp[strlen(temp)-1]='\0';
               if (strstr(temp,INFORMATION_END)==NULL) goto blad;

               if (glo_fgets(temp,200,stream)==NULL) goto blad;
               temp[strlen(temp)-1]='\0';
               if (strstr(temp,FACT_SIMPLE_END)==NULL) goto blad;

               // dopisanie faktu prostego do faktu
               if (fact->fs==NULL) fact->fs=fs;
               else fs_act->next=fs;
               fs_act=fs;
            }
            else if (strstr(temp,FACT_END)!=NULL) break;
            else goto blad;
         }
         // dopisanie faktu do tematu
         if (database->facts==NULL) database->facts=fact;
         else fact_act->next=fact;
         fact_act=fact;
      }
      else if (strcmp(temp,DATABASE_END)==0) break;
      else goto blad;

      if (glo_fgets(temp,200,stream)==NULL) goto blad;
      temp[strlen(temp)-1]='\0';
   }
   fclose(stream);

   // dodanie bazy faktow do bazy ogolnej
   if (handle->facts==NULL) handle->facts=database;
   else
   {
      database->next=handle->facts->next;
      handle->facts->next=database;
   }

   return TRUE;

   blad:
   printf("bladzio bladzio w database2.c!\n");
   if (stream) fclose(stream);
//   return FAIL;
   return TRUE;
}
// ***************************************************************************************
// EXTERN                    BOOL dat_facts_save(struct DeniseHandle*)
// ***************************************************************************************
BOOL dat_facts_save(struct DeniseHandle* handle)
{
   int nr1;
   char temp[1000];
   struct FactDatabase *database;
   struct Theme *theme,*theme_fwd;
   struct Fact *fact;
   struct FactSimple *fs;
   FILE *stream=NULL;

   if (handle->facts==NULL) return TRUE;

   // zapisujemy kolejne bazy faktow do odpowiednich plikow
   for (database=handle->facts;database!=NULL;database=database->next)
   {
      sprintf(temp,"%s%s%s.dat",handle->Prefix,handle->FactsDir,database->name);
      if ((stream=dat_file_open(handle,temp,"w"))==NULL) return FAIL;

      fprintf(stream,"%s\n",DATABASE_START);
      if (database->mode_name1!=NULL) fprintf(stream,"   %s\n",database->mode_name1);
      else fprintf(stream,"   %d\n",database->new_themes);
      if (database->mode_name2!=NULL) fprintf(stream,"   %s\n",database->mode_name2);
      else fprintf(stream,"   %d\n",database->new_facts);
      if (database->mode_name3!=NULL) fprintf(stream,"   %s\n",database->mode_name3);
      else fprintf(stream,"   %d\n",database->fwd_themes);
      fprintf(stream,"   %s\n",database->databaseID);

      // zapisujemy kolejne tematy w bazie faktow
      for (theme=database->themes;theme!=NULL;theme=theme->next)
      {
         fprintf(stream,"   %s\n",THEME_START);
         // identyfikatory
         if (theme->fwd_databaseID[0]=='\0')
         {
            if (theme->identify!=NULL) fprintf(stream,"      %s\n",theme->identify);
            else fprintf(stream,"      \n");
         }
         else
         {
            theme_fwd=ide_find_theme(handle,theme->fwd_databaseID,theme->fwd_number);
            if (theme_fwd!=NULL)
            {
               if (theme_fwd->identify!=NULL) fprintf(stream,"      %s\n",theme_fwd->identify);
               else fprintf(stream,"      \n");
            }
            else fprintf(stream,"      \n");
         }
         // numer identyfikacyjny
         fprintf(stream,"      %d\n",theme->number);
         // koniec tematu
         fprintf(stream,"   %s\n",THEME_END);
      }

      // zapisujemy kolejne fakty w temacie
      for (fact=database->facts;fact!=NULL;fact=fact->next)
      {
         fprintf(stream,"   %s\n",FACT_START);
         // zapisujemy czas i opis logiczny
         fprintf(stream,"      %s\n",fact->time);
         fprintf(stream,"      %d\n",fact->ile_klauzul);

         for (nr1=0;nr1<fact->ile_klauzul;nr1++)
            fprintf(stream,"      %s\n",fact->logic[nr1]);

         // zapisujemy kolejne fakty proste
         for (fs=fact->fs;fs!=NULL;fs=fs->next)
         {
            fprintf(stream,"      %s\n",FACT_SIMPLE_START);
            fprintf(stream,"         %d\n",fs->quality);
            fprintf(stream,"         %s\n",fs->databaseID);
            fprintf(stream,"         %d\n",fs->theme_number);
            if (fs->sentence)
            {
               fprintf(stream,"         %s\n",INFORMATION_START);
               fprintf(stream,"            %s\n",fs->sentence->language);
               fprintf(stream,"            %s\n",fs->sentence->id_subject);
               if (fs->sentence->id_global==NULL) fprintf(stream,"            \n");
               else fprintf(stream,"            %s\n",fs->sentence->id_global);
               fprintf(stream,"            %s\n",fs->sentence->orig);
               fprintf(stream,"            %s\n",fs->sentence->words);
               fprintf(stream,"            %s\n",fs->sentence->base_words);
               fprintf(stream,"            %s\n",fs->sentence->voc);
               if (fs->sentence->group_voc==NULL) fprintf(stream,"            \n");
					else fprintf(stream,"            %s\n",fs->sentence->group_voc);

               for (nr1=0;nr1<fs->sentence->words_number;nr1++) temp[nr1]='0'+fs->sentence->zrost_poz[nr1]; temp[nr1]='\0';
               fprintf(stream,"            %s\n",temp);

               for (nr1=0;nr1<fs->sentence->words_number;nr1++) temp[nr1]='0'+fs->sentence->priority[nr1]; temp[nr1]='\0';
               fprintf(stream,"            %s\n",temp);

               for (nr1=0;nr1<fs->sentence->words_number;nr1++) temp[nr1]='0'+fs->sentence->fleksyjny[nr1]; temp[nr1]='\0';
               fprintf(stream,"            %s\n",temp);

               for (nr1=0;nr1<fs->sentence->words_number;nr1++) temp[nr1]='0'+fs->sentence->member[nr1]; temp[nr1]='\0';
               fprintf(stream,"            %s\n",temp);

               for (nr1=0;nr1<fs->sentence->groups_number;nr1++) temp[nr1]='0'+fs->sentence->parent[nr1]; temp[nr1]='\0';
               fprintf(stream,"            %s\n",temp);

               for (nr1=0;nr1<fs->sentence->groups_number;nr1++) temp[nr1]='0'+fs->sentence->parent_word[nr1]; temp[nr1]='\0';
               fprintf(stream,"            %s\n",temp);

               for (nr1=0;nr1<fs->sentence->groups_number;nr1++) temp[nr1]='0'+fs->sentence->function[nr1]; temp[nr1]='\0';
               fprintf(stream,"            %s\n",temp);

               for (nr1=0;nr1<fs->sentence->groups_number;nr1++) temp[nr1]='0'+fs->sentence->posneg[nr1]; temp[nr1]='\0';
               fprintf(stream,"            %s\n",temp);

               fprintf(stream,"         %s\n",INFORMATION_END);
            }
            fprintf(stream,"      %s\n",FACT_SIMPLE_END);
         }
         fprintf(stream,"   %s\n",FACT_END);
      }
      fprintf(stream,"%s\n",DATABASE_END);
      fclose(stream);
   }
   return TRUE;
}

// ***************************************************************************************
// EXTERN                    BOOL dat_facts_erase(struct DeniseHandle*)
// ***************************************************************************************
BOOL dat_facts_erase(struct DeniseHandle* handle)
{
   struct FactDatabase *facts,*facts_next;

   if (handle->facts==NULL) return TRUE;

   // kasujemy kolejne bazy faktow
   for (facts=handle->facts;facts!=NULL;facts=facts_next)
   {
      facts_next=facts->next;
      dat_database_free(facts);
		free(facts);
   }
   handle->facts=NULL;

   return TRUE;
}

// ***************************************************************************************
// EXTERN                          void dat_database_free(database)
// ***************************************************************************************
void dat_database_free(struct FactDatabase *database)
{
   struct Theme *theme,*theme_next;
   struct Fact *fact,*fact_next;

   if (database->name!=NULL) free(database->name);
   if (database->mode_name1!=NULL) free(database->mode_name1);
   if (database->mode_name2!=NULL) free(database->mode_name2);
   if (database->mode_name3!=NULL) free(database->mode_name3);

   // kasujemy kolejne tematy w bazie faktow
   for (theme=database->themes;theme!=NULL;theme=theme_next)
   {
      theme_next=theme->next;
      if (theme->identify) free(theme->identify);
      free(theme);
   }

   // kasujemy kolejne fakty
   for (fact=database->facts;fact!=NULL;fact=fact_next)
   {
      fact_next=fact->next;
      dat_fact_free(fact);
		free(fact);
   }
//   free(database);
}

// ***************************************************************************************
// EXTERN                       void dat_fact_simple_free(fs)
// ***************************************************************************************
void dat_fact_simple_free(struct FactSimple *fs)
{
   if (fs==NULL) return;

   // kasujemy kolejne fakty proste

   if (fs->sentence)
	{
   	if (fs->sentence->language) free(fs->sentence->language);
      if (fs->sentence->id_subject) free(fs->sentence->id_subject);
      if (fs->sentence->id_global) free(fs->sentence->id_global);
      if (fs->sentence->orig) free(fs->sentence->orig);
      if (fs->sentence->words) free(fs->sentence->words);
      if (fs->sentence->base_words) free(fs->sentence->base_words);
      if (fs->sentence->voc) free(fs->sentence->voc);
      if (fs->sentence->group_voc) free(fs->sentence->group_voc);
      if (fs->sentence->zrost_poz) free(fs->sentence->zrost_poz);
      if (fs->sentence->priority) free(fs->sentence->priority);
      if (fs->sentence->fleksyjny) free(fs->sentence->fleksyjny);
      if (fs->sentence->member) free(fs->sentence->member);
      if (fs->sentence->parent) free(fs->sentence->parent);
      if (fs->sentence->parent_word) free(fs->sentence->parent_word);
      if (fs->sentence->function) free(fs->sentence->function);
      if (fs->sentence->posneg) free(fs->sentence->posneg);
      free(fs->sentence);
   }
//	free(fs);
}
// ***************************************************************************************
// EXTERN                          void dat_fact_free(fact)
// ***************************************************************************************
void dat_fact_free(struct Fact *fact)
{
   struct FactSimple *fs,*fs_next;
   int nr1;

   if (fact==NULL) return;

// fact->time jest statycznym stringiem   if (fact->time) free(fact->time);
   if (fact->logic)
   {
      for (nr1=0;nr1<fact->ile_klauzul;nr1++)
         if (fact->logic[nr1]) free(fact->logic[nr1]);
      free(fact->logic);
   }

   // kasujemy kolejne fakty proste
   for (fs=fact->fs;fs!=NULL;fs=fs_next)
   {
      fs_next=fs->next;
		dat_fact_simple_free(fs);
		free(fs);
   }
//   free(fact);
}
