/*
zmiany w kodzie do wersji 5.1 wprowadzili:
Szymon Jessa
Tomasz Cichocki
Stanislaw Szymczyk


w programowaniu wersji 5.0 wzieli udzial:
Szymon Jessa
Piotr Drapich
Piotr Radej


*/


/* Deniselib.c - by Peter Drapich docent@union.pl */

#include "denise.h"
#include "errors.c"

#ifdef PC_WIN
int WINAPI DllMain (HINSTANCE hInstance, DWORD fdwReason, PVOID pvReserved)
{
     return TRUE;
}
#endif

static char *prefix=NULL;
void SetPrefix(char *pref) {
	prefix=strdup(pref);
}

EXPORT struct DeniseHandle* CALLBACK DeniseOpenTalk(void(*print)(char* text),void(*report)(char* text))
{
	struct DeniseHandle* handle=NULL;
   char text[DENISE_SYS_BUFSIZE];
   char *configdirfile=NULL;

	if ((handle=UNN_AllocMem(sizeof(struct DeniseHandle),MEMF_PUBLIC|MEMF_CLEAR)))
	{
		handle->Print=print;
		handle->Report= report!=NULL ? report : print;
		if(prefix) strcpy(handle->Prefix,prefix);
		else strcpy(handle->Prefix,"");
		
		asprintf(&configdirfile,"%s/%s",handle->Prefix,CONFIG_FILE);
		com_load_config(handle,configdirfile);
		if(configdirfile) free(configdirfile);
		DeniseReport(handle,UNN_GetLocaleString(handle,MSG_PROGNAME,"Denise-MYTH %s"),DENISE_VERSION);
		glo_current_time(handle->Message,sizeof(handle->Message));
		if (dat_load_all(handle)==FAIL)
      {
			CloseCatalog(handle);
         UNN_FreeMem(handle);
         handle=NULL;
         return FAIL;
      }
	   if (handle->logbookparams.dialog==ON)
      {
			sprintf(text,UNN_GetLocaleString(handle,MSG_PROGNAME,"Denise-MYTH %s Artificial Intelligence"),DENISE_VERSION);
      	log_dialog(handle,text);
      }
		DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_WRITE_YOUR_NAME,"<musisz sie przedstawic: jestem Twoje_imie>"));
  		return handle;
	}
	return FAIL;
}


EXPORT int CALLBACK DeniseTalk(struct DeniseHandle* handle, char* textin, char* textout, int bufsize)
{
	char buffer[DENISE_SYS_BUFSIZE];

	if (handle)
	{
		if (textin)
      {
//		DeniseMessage(handle,"USER: %s",textin); // tekst rozmowcy na ekran
		if (handle->logbookparams.dialog==ON)
		{
			sprintf(buffer,"USER: %s",textin);
			log_dialog(handle,buffer);
		}
      	strncpy(handle->syntax.orig,textin,sizeof(handle->syntax.orig)-1);
      }
		handle->output[0]='\0';
		myth(handle);
		if (textout) strncpy(textout,handle->output, bufsize);
//		printf("!%s!\n",textout);
//		printf("*%s*\n",handle->output);
//      sprintf(textout,"ready");
		return DONE;
	}
	return ERR_SYS_INVALIDHANDLE;
}


EXPORT int CALLBACK DeniseCloseTalk(struct DeniseHandle* handle)
{
	if (handle)
	{
		DeniseReport(handle,UNN_GetLocaleString(handle,MSG_EXITING,"Wychodz z programu."));
		dat_save_all(handle);
		dat_erase_all(handle);
		UNN_FreeMem(handle);
      handle=NULL;
		return DONE;
	}
	return ERR_SYS_INVALIDHANDLE;
}


EXPORT int CALLBACK DeniseSet(struct DeniseHandle* handle, int what, void* data)
{
	if (handle)
	{
		switch (what)
		{
			case DENISE_CMD_MINWORDS:
				if (!strcmp((char *) data,"off")) handle->syntaxparams.min_words=0;
				else handle->syntaxparams.min_words=max(0,atoi((char *) data));
				break;
			case DENISE_CMD_MINGROUPS:
				if (!strcmp((char *) data,"off")) handle->syntaxparams.min_groups=0;
				else handle->syntaxparams.min_groups=max(0,atoi((char *) data));
				break;
			case DENISE_CMD_MAXGROUPS:
				if (!strcmp((char *) data,"off")) handle->syntaxparams.max_groups=0;
				else handle->syntaxparams.max_groups=max(0,atoi((char *) data));
				break;
			case DENISE_CMD_MAXFLEKS:
				if (!strcmp((char *) data,"off")) handle->syntaxparams.max_fleks=0;
				else handle->syntaxparams.max_fleks=max(0,atoi((char *) data));
				break;
			case DENISE_CMD_MAXWORDS:
				if (!strcmp((char *) data,"off")) handle->syntaxparams.max_words=0;
				else handle->syntaxparams.max_words=max(0,atoi((char *) data));
				break;
			case DENISE_CMD_MAXCLAUSES:
				if (!strcmp((char *) data,"off")) handle->syntaxparams.max_clauses=0;
				else handle->syntaxparams.max_clauses=max(0,atoi((char *) data));
				break;
			case DENISE_CMD_FIRSTCLAUSE:
				handle->syntaxparams.first_clause=(int)data==DENISE_CMD_ON?ON:OFF;
				break;
			case DENISE_CMD_SUBJECTFIRST:
				handle->syntaxparams.subject_first=(int)data==DENISE_CMD_ON?ON:OFF;
				break;
			case DENISE_CMD_MINPREFIXLEN:
				handle->syntaxparams.min_prefix_len=max(1,atoi((char *) data));
				break;
			case DENISE_CMD_PREFIXCHECK:
				handle->syntaxparams.prefix_check=(int)data==DENISE_CMD_ON?ON:OFF;
				break;
			case DENISE_CMD_PREFIXCHECKMODE:
				handle->syntaxparams.prefix_check_mode=min(1,max(0,atoi((char *) data)));
				break;
			case DENISE_CMD_REMEMBERFIRST:
				handle->syntaxparams.remember_first=(int)data==DENISE_CMD_ON?ON:OFF;
				break;
			case DENISE_CMD_PHRASEQUALITY:
				handle->syntaxparams.phrase_quality=min(100,max(0,atoi((char*) data)));
				break;
			case DENISE_CMD_STRUCTUREQUALITY:
				handle->syntaxparams.structure_quality=min(100,max(0,atoi((char*) data)));
				break;
			case DENISE_CMD_FUNCTIONQUALITY:
				handle->syntaxparams.function_quality=min(100,max(0,atoi((char*) data)));
				break;
			case DENISE_CMD_VIEWWORDS:
				handle->viewparams.words=(int)data==DENISE_CMD_ON?ON:OFF;
				break;
			case DENISE_CMD_VIEWCLAUSE:
				handle->viewparams.clause=(int)data==DENISE_CMD_ON?ON:OFF;
				break;
			case DENISE_CMD_VIEWSYNTAX:
				handle->viewparams.syntax=(int)data==DENISE_CMD_ON?ON:OFF;
				break;
			case DENISE_CMD_VIEWSYNTAXTIME:
				handle->viewparams.syntax_time=(int)data==DENISE_CMD_ON?ON:OFF;
				break;
			case DENISE_CMD_VIEWPRELIMINARY:
				handle->viewparams.preliminary_analysis=(int)data==DENISE_CMD_ON?ON:OFF;
				break;
			case DENISE_CMD_VIEWTHEME:
				handle->viewparams.theme_analysis=(int)data==DENISE_CMD_ON?ON:OFF;
				break;
			case DENISE_CMD_VIEWINTERLOCUTORY:
				handle->viewparams.interlocutory_analysis=(int)data==DENISE_CMD_ON?ON:OFF;
				break;
			case DENISE_CMD_VIEWLOGIC:
				handle->viewparams.logic_analysis=(int)data==DENISE_CMD_ON?ON:OFF;
				break;
			case DENISE_CMD_VIEWFACT:
				handle->viewparams.fact_analysis=(int)data==DENISE_CMD_ON?ON:OFF;
				break;
			case DENISE_CMD_VIEWUNKNOWNWORDS:
				handle->viewparams.unknown_words=(int)data==DENISE_CMD_ON?ON:OFF;
				break;
			case DENISE_CMD_VIEWCONJUNCTIONSANALYSIS:
				handle->viewparams.conjunctions_analysis=(int)data==DENISE_CMD_ON?ON:OFF;
				break;
			case DENISE_CMD_LOGWORDS:
				handle->logbookparams.words=(int)data==DENISE_CMD_ON?ON:OFF;
				break;
			case DENISE_CMD_LOGCLAUSES:
				handle->logbookparams.clauses=(int)data==DENISE_CMD_ON?ON:OFF;
				break;
			case DENISE_CMD_LOGDIALOG:
				handle->logbookparams.dialog=(int)data==DENISE_CMD_ON?ON:OFF;
				break;
			case DENISE_CMD_LOGREPORTS:
				handle->logbookparams.reports=(int)data==DENISE_CMD_ON?ON:OFF;
				break;
			case DENISE_CMD_SUPERVISOR:
				handle->Supervisor=(int)data==DENISE_CMD_ON?ON:OFF;
				break;
			case DENISE_CMD_LANGUAGE:
				DeniseReport(handle,UNN_GetLocaleString(handle,MSG_SELECTED_LANGUAGE,"aktualny jezyk: %s"),(char *) data);
				dat_language_erase(handle);
				handle->language.name=malloc(strlen((char *) data)+1);
				memcpy(handle->language.name,(char *) data,strlen((char *) data)+1);
				dat_language_load(handle);
				break;
		}
	} else return ERR_SYS_INVALIDHANDLE;
	return DONE;
}


EXPORT int CALLBACK DeniseGet(struct DeniseHandle* handle, int what, void* data)
{
	if (handle)
	{
		switch (what)
		{
			case DENISE_CMD_MINWORDS:
				if (handle->syntaxparams.min_words==0) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_NOT_DEFINED,"undefined"));
				else DeniseReport(handle,"minwords = %d",handle->syntaxparams.min_words);
				break;
			case DENISE_CMD_MINGROUPS:
				if (handle->syntaxparams.min_groups==0) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_NOT_DEFINED,"undefined"));
				else DeniseReport(handle,"mingroups =	%d",handle->syntaxparams.min_groups);
				break;
			case DENISE_CMD_MAXGROUPS:
				if (handle->syntaxparams.max_groups==0) DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_NOT_DEFINED,"undefined"));
				else DeniseMessage(handle,"maxgroups = %d",handle->syntaxparams.max_groups);
				break;
			case DENISE_CMD_MAXFLEKS:
				if (handle->syntaxparams.max_fleks==0) DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_NOT_DEFINED,"undefined"));
				else DeniseMessage(handle,"maxfleks = %d",handle->syntaxparams.max_fleks);
				break;
			case DENISE_CMD_MAXWORDS:
				if (handle->syntaxparams.max_words==0) DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_NOT_DEFINED,"undefined"));
				else DeniseMessage(handle,"maxwords = %d",handle->syntaxparams.max_words);
				break;
			case DENISE_CMD_MAXCLAUSES:
				if (handle->syntaxparams.max_clauses==0) DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_NOT_DEFINED,"undefined"));
				else DeniseMessage(handle,"maxclauses = %d",handle->syntaxparams.max_clauses);
				break;
			case DENISE_CMD_FIRSTCLAUSE:
				if (handle->syntaxparams.first_clause==ON) DeniseMessage(handle,"firstclause = ON");
				else DeniseMessage(handle,"firstclause = OFF");
				break;
			case DENISE_CMD_SUBJECTFIRST:
				if (handle->syntaxparams.subject_first==ON) DeniseMessage(handle,"subjectfirst = ON");
				else DeniseMessage(handle,"subjectfirst = OFF");
				break;
			case DENISE_CMD_MINPREFIXLEN:
				DeniseMessage(handle,"minprefixlen = %d",handle->syntaxparams.min_prefix_len);
				break;
			case DENISE_CMD_PREFIXCHECK:
				if (handle->syntaxparams.prefix_check==ON) DeniseMessage(handle,"prefixcheck = ON");
				else DeniseMessage(handle,"prefixcheck = OFF");
				break;
			case DENISE_CMD_PREFIXCHECKMODE:
				DeniseMessage(handle,"prefixcheckmode = %d",handle->syntaxparams.prefix_check_mode);
				break;
			case DENISE_CMD_REMEMBERFIRST:
				if (handle->syntaxparams.remember_first==ON) DeniseMessage(handle,"rememberfirst = ON");
				else DeniseMessage(handle,"rememberfirst = OFF");
				break;
			case DENISE_CMD_PHRASEQUALITY:
				DeniseMessage(handle,"phrasequality = %d%%",handle->syntaxparams.phrase_quality);
				break;
			case DENISE_CMD_STRUCTUREQUALITY:
				DeniseMessage(handle,"structurequality = %d%%",handle->syntaxparams.structure_quality);
				break;
			case DENISE_CMD_FUNCTIONQUALITY:
				DeniseMessage(handle,"functionquality = %d%%",handle->syntaxparams.function_quality);
				break;
			case DENISE_CMD_VIEWWORDS:
				if (handle->viewparams.words==ON) DeniseMessage(handle,"viewwords = ON");
				else DeniseMessage(handle,"viewwords = OFF");
				break;
			case DENISE_CMD_VIEWCLAUSE:
				if (handle->viewparams.clause==ON) DeniseMessage(handle,"viewclause = ON");
				else DeniseMessage(handle,"viewclause = OFF");
				break;
			case DENISE_CMD_VIEWSYNTAX:
				if (handle->viewparams.syntax==ON) DeniseMessage(handle,"viewsyntax = ON");
				else DeniseMessage(handle,"viewsyntax = OFF");
				break;
			case DENISE_CMD_VIEWSYNTAXTIME:
				if (handle->viewparams.syntax_time==ON) DeniseMessage(handle,"viewsyntaxtime = ON");
				else DeniseMessage(handle,"viewsyntaxtime = OFF");
				break;
			case DENISE_CMD_VIEWPRELIMINARY:
				if (handle->viewparams.preliminary_analysis==ON) DeniseMessage(handle,"viewpreliminaryanalysis = ON");
				else DeniseMessage(handle,"viewpreliminaryanalysis = OFF");
				break;
			case DENISE_CMD_VIEWTHEME:
				if (handle->viewparams.theme_analysis==ON) DeniseMessage(handle,"viewthemeanalysis = ON");
				else DeniseMessage(handle,"viewthemeanalysis = OFF");
				break;
			case DENISE_CMD_VIEWINTERLOCUTORY:
				if (handle->viewparams.interlocutory_analysis==ON) DeniseMessage(handle,"viewinterlocutoryanalysis = ON");
				else DeniseMessage(handle,"viewinterlocutoryanalysis = OFF");
				break;
			case DENISE_CMD_VIEWLOGIC:
				if (handle->viewparams.logic_analysis==ON) DeniseMessage(handle,"viewlogicanalysis = ON");
				else DeniseMessage(handle,"viewlogicanalysis = OFF");
				break;
			case DENISE_CMD_VIEWFACT:
				if (handle->viewparams.fact_analysis==ON) DeniseMessage(handle,"viewfactanalysis = ON");
				else DeniseMessage(handle,"viewfactanalysis = OFF");
				break;
			case DENISE_CMD_VIEWUNKNOWNWORDS:
				if (handle->viewparams.unknown_words==ON) DeniseMessage(handle,"viewunknownwords = ON");
				else DeniseMessage(handle,"viewunknownwords = OFF");
				break;
			case DENISE_CMD_VIEWCONJUNCTIONSANALYSIS:
				if (handle->viewparams.conjunctions_analysis==ON) DeniseMessage(handle,"viewconjunctionsanalysis = ON");
				else DeniseMessage(handle,"viewconjunctionsanalysis = OFF");
				break;
			case DENISE_CMD_LOGWORDS:
				if (handle->logbookparams.words==ON) DeniseMessage(handle,"logwords = ON");
				else DeniseMessage(handle,"logwords = OFF");
				break;
			case DENISE_CMD_LOGCLAUSES:
				if (handle->logbookparams.clauses==ON) DeniseMessage(handle,"logclauses = ON");
				else DeniseMessage(handle,"logclauses = OFF");
				break;
			case DENISE_CMD_LOGDIALOG:
				if (handle->logbookparams.dialog==ON) DeniseMessage(handle,"logdialog = ON");
				else DeniseMessage(handle,"logdialog = OFF");
				break;
			case DENISE_CMD_LOGREPORTS:
				if (handle->logbookparams.reports==ON) DeniseMessage(handle,"logreports = ON");
				else DeniseMessage(handle,"logreports = OFF");
				break;
			case DENISE_CMD_SUPERVISOR:
				if (handle->Supervisor==ON) DeniseMessage(handle,"supervisor = ON");
				else DeniseMessage(handle,"supervisor = OFF");
				break;
			case DENISE_CMD_LANGUAGE:
				DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_SELECTED_LANGUAGE,"aktualny jezyk: %s"),handle->language.name);
				break;
		}
	}
	return ERR_SYS_INVALIDHANDLE;
}


EXPORT int CALLBACK DeniseLoad(struct DeniseHandle* handle, int what, void* data)
{
	if (handle)
	{
		switch (what)
		{
			case DENISE_CMD_LOADCONFIG:
				com_load_config(handle,(char *) data);
				break;
		}
	} else return ERR_SYS_INVALIDHANDLE;
	return DONE;
}


EXPORT int CALLBACK DeniseSave(struct DeniseHandle* handle, int what, void* data)
{
	if (handle)
	{
		switch (what)
		{
			case DENISE_CMD_SAVECONFIG:
				com_save_config(handle,(char *) data);
				break;
		}
	} else return ERR_SYS_INVALIDHANDLE;
	return DONE;
}


int DeniseLoadKnowledge(struct DeniseHandle* handle, int what, char* filename)
{
	if (handle)
	{
		if (filename)
		{
/*			switch (what)
			{
				case DENISE_
			}
*/		}
	}
	return ERR_SYS_INVALIDHANDLE;
}


EXPORT char* CALLBACK DeniseError(struct DeniseHandle* handle, int errorcode)
{
	if (handle)
	{
		if (errorcode>DENISE_MAX_ERRORS) errorcode=DENISE_MAX_ERRORS;
		if (errorcode) return(DeniseErrorTable[errorcode]);
		else return(DeniseErrorTable[handle->ErrorCode]);
	}
	return DeniseErrorTable[ERR_SYS_INVALIDHANDLE];
}


EXPORT char* CALLBACK DeniseWarning(struct DeniseHandle* handle, int warncode)
{
	if (handle)
	{
		if (warncode>DENISE_MAX_ERRORS) warncode=DENISE_MAX_ERRORS;
		if (warncode) return(DeniseErrorTable[warncode]);
		else return(DeniseErrorTable[handle->WarnCode]);
	}
	return DeniseErrorTable[ERR_SYS_INVALIDHANDLE];
}

#ifndef UNIX
EXPORT int STDARGS CALLBACK DeniseMessage(struct DeniseHandle* handle, const char *format, ...)
#else
int DeniseMessage(struct DeniseHandle* handle, const char *format, ...)
#endif 
{
//	static char last[DENISE_SYS_BUFSIZE];
	char buffer[DENISE_SYS_BUFSIZE],ln[5];
	va_list arglist;

	va_start(arglist,format);
	vsprintf(buffer, format, arglist);
	va_end(arglist);

   // zeby nie wyswietlac kilka razy tej samej informacji linia po linii
//  	if (strcmp(last,buffer)==0) return DONE;
//	else strcpy(last,buffer);

   if (handle->logbookparams.dialog==ON) log_dialog(handle,buffer);

	sprintf(ln,"\n");

	if (handle)
	{
		strncpy(handle->Message,buffer,sizeof(handle->Message)-1);
		strncat(handle->output,ln,sizeof(handle->Message)-1-strlen(handle->output));
		strncat(handle->output,buffer,sizeof(handle->Message)-1-strlen(handle->output));
		if (handle->Print) handle->Print(buffer);
		return DONE;
	}

	return ERR_SYS_INVALIDHANDLE;
}

#ifndef UNIX
EXPORT int STDARGS CALLBACK DeniseReport(struct DeniseHandle* handle, const char *format, ...)
#else
int DeniseReport(struct DeniseHandle* handle, const char *format, ...)
#endif 
{
	static char last[DENISE_SYS_BUFSIZE];
	char buffer[DENISE_SYS_BUFSIZE],ln[5];
	va_list arglist;

	va_start(arglist,format);
	vsprintf(buffer, format, arglist);
	va_end(arglist);

   // zeby nie wyswietlac kilka razy tej samej informacji linia po linii
  	if (strcmp(last,buffer)==0) return DONE;
	else strcpy(last,buffer);

   if (handle->logbookparams.reports==ON) log_reports(handle,buffer);

	sprintf(ln,"\n");

	if (handle)
	{
		strncpy(handle->Message,buffer,sizeof(handle->Message)-1);
		strncat(handle->output,ln,sizeof(handle->Message)-1-strlen(handle->output));
		strncat(handle->output,buffer,sizeof(handle->Message)-1-strlen(handle->output));
		if (handle->Report) handle->Report(buffer);
		return DONE;
	}

	return ERR_SYS_INVALIDHANDLE;
}

